import { DEFAULT_MAX_CONCURRENT_REQUESTS, StatusEnum } from './constants.js';

class DataLoader {
  static maxConcurrent = DEFAULT_MAX_CONCURRENT_REQUESTS;
  static started = 0;
  static cachedData = {};
  static queue = {};
  isCalled = false;
  isCancelled = true;
  status = StatusEnum.IDLE;
  observers = [];
  loadCount = 0;
  isFirstLoading = true;
  constructor(args) {
    this.key = args.key;
    this.method = args.method;
    if (args.enabled) {
      this.status = StatusEnum.LOADING;
    }
    this.data = DataLoader.cachedData[this.key];
    if (args.notifyChanges) this.observers.push(args.notifyChanges);
  }
  notifyChanges() {
    if (this.timeout) {
      clearTimeout(this.timeout);
    }
    this.timeout = setTimeout(() => {
      this.observers.forEach(observer => observer());
    });
  }
  getData() {
    return DataLoader.cachedData[this.key];
  }
  tryLaunch = async () => {
    if (DataLoader.started < DataLoader.maxConcurrent) {
      DataLoader.started += 1;
      DataLoader.queue[this.key] = this.launch();
    } else {
      DataLoader.queue[this.key] = new Promise(resolve => {
        setTimeout(resolve);
      }).then(this.tryLaunch);
    }
    return DataLoader.queue[this.key];
  };
  load = (() => {
    var _this = this;
    return async function () {
      let force = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
      if (force || !_this.isCalled) {
        _this.isCalled = true;
        if (_this.status !== StatusEnum.LOADING) {
          _this.status = StatusEnum.LOADING;
          _this.notifyChanges();
        }
        DataLoader.queue[_this.key] = _this.tryLaunch();
      }
      return DataLoader.queue[_this.key];
    };
  })();
  launch = async () => {
    try {
      this.isCancelled = false;
      this.loadCount += 1;
      const data = await this.method();
      if (!this.isCancelled) {
        DataLoader.cachedData[this.key] = data;
        this.status = StatusEnum.SUCCESS;
        this.data = data;
        this.error = undefined;
        this.dataUpdatedAt = Date.now();
      }
      this.isCalled = false;
      this.isFirstLoading = false;
      DataLoader.started -= 1;
      delete DataLoader.queue[this.key];
      this.notifyChanges();
      return data;
    } catch (error) {
      if (!this.isCancelled) {
        this.status = StatusEnum.ERROR;
        this.error = error;
      }
      this.isCalled = false;
      this.isFirstLoading = false;
      DataLoader.started -= 1;
      delete DataLoader.queue[this.key];
      this.notifyChanges();
      if (!this.isCancelled) {
        throw error;
      }
      return undefined;
    }
  };
  clearData() {
    DataLoader.cachedData[this.key] = undefined;
  }
  cancel() {
    DataLoader.started -= 1;
    delete DataLoader.queue[this.key];
    this.isCancelled = true;
    this.status = StatusEnum.IDLE;
  }
  addObserver(observer) {
    this.observers.push(observer);
  }
  removeObserver(observer) {
    const index = this.observers.indexOf(observer);
    if (index > -1) this.observers.splice(index, 1);
  }
}
var DataLoader$1 = DataLoader;

export { DataLoader$1 as default };
