import { ReactElement, ReactNode } from 'react';

declare enum StatusEnum {
    ERROR = "error",
    IDLE = "idle",
    LOADING = "loading",
    SUCCESS = "success"
}

type PrimitiveType = string | number | boolean | null | undefined | Date;
type KeyType = string | number | PrimitiveType[];
declare class PromiseType<T = unknown> extends Promise<T> {
    cancel?: () => void;
}
type OnErrorFn<ErrorType = Error> = ((err: ErrorType) => void | Promise<void>) | undefined;
type OnSuccessFn<ResultType> = ((result: ResultType) => void | Promise<void>) | undefined;
type NeedPollingType<ResultType> = boolean | ((data?: ResultType) => boolean);
/**
 * @typedef {Object} UseDataLoaderConfig
 * @property {Function} [onSuccess] callback when a request success
 * @property {Function} [onError] callback when a error is occured, this will override the onError specified on the Provider if any
 * @property {*} [initialData] initial data if no one is present in the cache before the request
 * @property {number} [pollingInterval] relaunch the request after the last success
 * @property {boolean} [enabled=true] launch request automatically (default true)
 * @property {boolean} [keepPreviousData=true] do we need to keep the previous data after reload (default true)
 * @property {number} [dataLifetime=undefined]  Time before data from previous success is considered as outdated (in millisecond)
 * @property {NeedPollingType} [needPolling=true] When pollingInterval is set you can set a set a custom callback to know if polling is enabled
 */
type UseDataLoaderConfig<ResultType, ErrorType> = {
    enabled?: boolean;
    initialData?: ResultType;
    keepPreviousData?: boolean;
    onError?: OnErrorFn<ErrorType>;
    onSuccess?: OnSuccessFn<ResultType>;
    pollingInterval?: number;
    dataLifetime?: number;
    needPolling?: NeedPollingType<ResultType>;
};
/**
 * @typedef {Object} UseDataLoaderResult
 * @property {boolean} isIdle true if the hook in initial state
 * @property {boolean} isLoading true if the request is launched
 * @property {boolean} isSuccess true if the request success
 * @property {boolean} isError true if the request throw an error
 * @property {boolean} isPolling true if the request if enabled is true, pollingInterval is defined and the status is isLoading or isSuccess
 * @property {*} previousData if keepPreviousData is true it return the last data fetched
 * @property {*} data initialData if no data is fetched or not present in the cache otherwise return the data fetched
 * @property {string} error the error occured during the request
 * @property {Function} reload reload the data
 */
type UseDataLoaderResult<ResultType, ErrorType> = {
    data?: ResultType;
    error?: ErrorType;
    isError: boolean;
    isIdle: boolean;
    isLoading: boolean;
    isPolling: boolean;
    isSuccess: boolean;
    previousData?: ResultType;
    reload: () => Promise<void>;
};
/**
 * Params send to the method
 */
type UsePaginatedDataLoaderMethodParams = {
    page: number;
    perPage: number;
};
type UsePaginatedDataLoaderConfig<ResultType, ErrorType> = UseDataLoaderConfig<ResultType, ErrorType> & {
    initialPage?: number;
    perPage?: number;
};
type UsePaginatedDataLoaderResult<ResultType, ErrorType> = {
    pageData?: ResultType;
    data?: Record<number, ResultType | undefined>;
    error?: ErrorType;
    isError: boolean;
    isIdle: boolean;
    isLoading: boolean;
    isPolling: boolean;
    isSuccess: boolean;
    reload: () => Promise<void>;
    goToPage: (page: number) => void;
    goToNextPage: () => void;
    goToPreviousPage: () => void;
    page: number;
};

type DataLoaderConstructorArgs<ResultType> = {
    key: string;
    method: () => PromiseType<ResultType>;
    enabled?: boolean;
    notifyChanges?: () => void;
};
declare class DataLoader<ResultType, ErrorType> {
    static maxConcurrent: number;
    static started: number;
    static cachedData: Record<string, unknown | undefined>;
    static queue: Record<string, PromiseType>;
    key: string;
    method: () => PromiseType<ResultType>;
    isCalled: boolean;
    isCancelled: boolean;
    status: StatusEnum;
    error?: ErrorType;
    data?: ResultType;
    observers: (() => void)[];
    timeout?: number;
    loadCount: number;
    isFirstLoading: boolean;
    dataUpdatedAt?: number;
    constructor(args: DataLoaderConstructorArgs<ResultType>);
    notifyChanges(): void;
    getData(): ResultType | undefined;
    private tryLaunch;
    load: (force?: boolean) => Promise<ResultType>;
    launch: () => Promise<ResultType | undefined>;
    clearData(): void;
    cancel(): void;
    addObserver(observer: () => void): void;
    removeObserver(observer: () => void): void;
}

type CachedData = Record<string, unknown>;
type Reloads = Record<string, () => Promise<void | unknown>>;
type UseDataLoaderInitializerArgs<ResultType = unknown> = {
    method: () => PromiseType<ResultType>;
    /**
     * Max time before data from previous success is considered as outdated (in millisecond)
     */
    maxDataLifetime?: number;
    enabled?: boolean;
};
type GetCachedDataFn = {
    (): CachedData;
    (key?: string): unknown | undefined;
};
type GetReloadsFn = {
    (): Reloads;
    (key?: string): (() => Promise<void | unknown>) | undefined;
};
type IDataLoaderContext = {
    addRequest: <ResultType, ErrorType>(key: string, args: UseDataLoaderInitializerArgs<ResultType>) => DataLoader<ResultType, ErrorType>;
    getOrAddRequest: <ResultType, ErrorType>(key: string, args: UseDataLoaderInitializerArgs<ResultType>) => DataLoader<ResultType, ErrorType>;
    cacheKeyPrefix?: string;
    onError?: (error: Error) => void | Promise<void>;
    clearAllCachedData: () => void;
    clearCachedData: (key: string) => void;
    getCachedData: GetCachedDataFn;
    getReloads: GetReloadsFn;
    getRequest: <ResultType, ErrorType>(key: string) => DataLoader<ResultType, ErrorType>;
    reload: (key?: string) => Promise<void>;
    reloadAll: () => Promise<void>;
};
type DataLoaderProviderProps = {
    children: ReactNode;
    cacheKeyPrefix?: string;
    onError?: OnErrorFn;
    maxConcurrentRequests?: number;
};
declare const DataLoaderProvider: ({ children, cacheKeyPrefix, onError, maxConcurrentRequests, }: DataLoaderProviderProps) => ReactElement;
declare const useDataLoaderContext: () => IDataLoaderContext;

declare function useDataLoader<ResultType = unknown, ErrorType = Error>(key: KeyType, method: () => PromiseType<ResultType>, { enabled, onError, onSuccess, keepPreviousData, needPolling, pollingInterval, initialData, dataLifetime, }?: UseDataLoaderConfig<ResultType, ErrorType>): UseDataLoaderResult<ResultType, ErrorType>;

/**
 * @param {KeyType} key base key used to cache data. Hook append -page-X to that key for each page you load
 * @param {() => PromiseType} method a method that return a promise
 * @param {useDataLoaderConfig} config hook configuration
 * @returns {useDataLoaderResult} hook result containing data, request state, and method to reload the data
 */
declare const usePaginatedDataLoader: <ResultType = unknown, ErrorType = Error>(key: KeyType, method: (params: UsePaginatedDataLoaderMethodParams) => PromiseType<ResultType>, { enabled, initialData, keepPreviousData, onError, onSuccess, pollingInterval, dataLifetime, needPolling, initialPage, perPage, }?: UsePaginatedDataLoaderConfig<ResultType, ErrorType>) => UsePaginatedDataLoaderResult<ResultType, ErrorType>;

export { DataLoaderProvider, UseDataLoaderConfig, useDataLoader, useDataLoaderContext, usePaginatedDataLoader };
