import { useRef, useState, useCallback, useMemo, useEffect } from 'react';
import { useDataLoaderContext } from './DataLoaderProvider.js';
import { StatusEnum } from './constants.js';
import { marshalQueryKey } from './helpers.js';

function useDataLoader(key, method) {
  let {
    enabled = true,
    onError,
    onSuccess,
    keepPreviousData = false,
    needPolling = true,
    pollingInterval,
    initialData,
    dataLifetime
  } = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  const {
    getOrAddRequest,
    onError: onGlobalError
  } = useDataLoaderContext();
  const methodRef = useRef(method);
  const onSuccessRef = useRef(onSuccess);
  const onErrorRef = useRef(onError ?? onGlobalError);
  const needPollingRef = useRef(needPolling);
  const [, setCounter] = useState(0);
  const forceRerender = useCallback(() => {
    setCounter(current => current + 1);
  }, []);
  const queryKey = useMemo(() => marshalQueryKey(key), [key]);
  const request = getOrAddRequest(queryKey, {
    enabled,
    method: methodRef.current
  });
  useEffect(() => {
    request.addObserver(forceRerender);
    return () => {
      request.removeObserver(forceRerender);
    };
  }, [request, forceRerender]);
  const needLoad = useMemo(() => !!(enabled && (!request.dataUpdatedAt || !dataLifetime || request.dataUpdatedAt && dataLifetime && request.dataUpdatedAt + dataLifetime < Date.now())), [enabled, request.dataUpdatedAt, dataLifetime]);
  const optimisticIsLoadingRef = useRef(needLoad);
  const previousDataRef = useRef(request.data);
  const isLoading = request.status === StatusEnum.LOADING || optimisticIsLoadingRef.current;
  const isSuccess = request.status === StatusEnum.SUCCESS;
  const isError = request.status === StatusEnum.ERROR;
  const isIdle = request.status === StatusEnum.IDLE && !enabled;
  const isPolling = !!(pollingInterval && (typeof needPolling === 'function' && (request.isFirstLoading || needPolling(request.data)) || typeof needPolling !== 'function' && needPolling));
  const reload = useCallback(() => request.load(true).then(onSuccessRef.current).catch(onErrorRef.current), [request]);
  useEffect(() => {
    needPollingRef.current = needPolling;
  }, [needPolling]);
  useEffect(() => {
    request.method = method;
  }, [method, request]);
  useEffect(() => {
    onSuccessRef.current = onSuccess;
  }, [onSuccess]);
  useEffect(() => {
    onErrorRef.current = onError ?? onGlobalError;
  }, [onError, onGlobalError]);
  useEffect(() => {
    if (keepPreviousData) {
      previousDataRef.current = request.data;
    }
  }, [request.data, keepPreviousData]);
  useEffect(() => {
    if (needLoad) {
      request.load().then(onSuccessRef.current).catch(onErrorRef.current);
    }
    optimisticIsLoadingRef.current = false;
  }, [needLoad, request]);
  useEffect(() => {
    let interval;
    if (pollingInterval) {
      interval = setInterval(() => {
        if (needPollingRef.current && typeof needPollingRef.current === 'function' && needPollingRef.current(request.data) || typeof needPollingRef.current !== 'function' && needPollingRef.current && !request.isCalled) {
          request.load(true).then(onSuccessRef.current).catch(onErrorRef.current);
        }
      }, pollingInterval);
    }
    return () => {
      if (interval) clearInterval(interval);
    };
  }, [pollingInterval, request]);
  return {
    data: !request.isFirstLoading ? request.data : initialData,
    error: request.error,
    isError,
    isIdle,
    isLoading,
    isPolling,
    isSuccess,
    previousData: previousDataRef.current,
    reload
  };
}

export { useDataLoader as default };
